#include "mbed.h"
#include "func.h"
#include "myClasses.h"
#include <vector>
#include <algorithm>

//---------------- SETUP UZLU--------------------//
char IDMaster = '0';                //ID tohoto mastera
int dobaLokalizacniSekvence = 500;  //ms
int pocetSlaveVezi = 1;
//-----------------------------------------------//


//---------------- PROMENNE----------------------//
Timer timerGeneralError;

//IQRF
SPI spi(D11, D12, D13);             // mosi, miso, sclk
DigitalOut slave(A3);
InterruptIn iqrfInterrupt(D7);
bool IQRFData = false;              //zda ma IQRF modul pripravena data

//seriova linka
Serial pc(USBTX, USBRX);
//Serial pc(D1, D0);

//LED
DigitalOut redLED(A2);
DigitalOut blueLED(A1);
DigitalOut greenLED(A0);

//ring
InterruptIn previous(D8);
DigitalOut next(D9);
bool pulzVObehu = false;
bool vysilamSekvenci = false;
Timer timerVysilaniSekvence;

//obsluha robotu
int aktualniRobot;                  //pozice ve vektoru roboti
bool oslovitRobota = false;         //zda mohu oslovit robota
vector<StatusRobota> robotiStatus;  //seznam vsech robotu a jejich statusu
Ticker tickerOslovitRobota;         //ticker pro oslovovani robotu
bool robotChcePolohy = false;       //zda chce nejaky robot polohy vsechn robotu
bool polohaPoslana = false;         //zda jsem vyslal polohu a cekam na odpoved
bool prenosPolohInit = false;       //zda mam poslal startovaci zpravu pro prenos poloh
char IDRobotaCoChcePolohy = 0;       //jaky robot chce polohy
int polohPoslano = 0;               //kolik poloh jsem jiz poslal
Timer timerOdesilaniPoloh;
bool najitDostupneRoboty = true;
bool najitDostupneRobotyInit = true;
int najitDostupneRobotyCnt = 0;
bool cekamNaOdpovedOdRobota = false;
//-----------------------------------------------//


//IQRF modul ma pripravena data
void iqrfHasData()
{
    IQRFData = true;
}

//prisel pulz od predchozi slave veze
void pulseReceived()
{
    pulzVObehu = false;
    timerVysilaniSekvence.stop();
    timerVysilaniSekvence.reset();
}

//prisel cas oslovit robota
void OslovitRobota()
{
    oslovitRobota = true;
    najitDostupneRobotyCnt++;
    if(najitDostupneRobotyCnt>20) {
        najitDostupneRobotyCnt=0;
        najitDostupneRobotyInit = true;
        najitDostupneRoboty = true;
    }
}

int main()
{
    aktualniRobot = 0;                                  //pozice aktualniho robota v seznamu robotu
    next = 0;                                           //nasledujici slave vez log. 0 - pro sychr

    //iniciace tickeru, interrupt pri prijmu dat od IQRF modulu a od predchozi slave veze
    tickerOslovitRobota.attach(&OslovitRobota, 1.0);    //frekvence oslovovani robotu
    iqrfInterrupt.rise(&iqrfHasData);                   //interrupt od IQRF modulu
    previous.rise(&pulseReceived);                      //interrupt od predchazejici slave veze

    //SPI setup
    wait(1);
    slave = 1;
    wait(0.1);
    spi.format(8,0);                                    // Setup:  8bitu, pol = 0, pha = 0 -> rising edge, idle low
    spi.frequency(100000);                              //Hz

    //IQRF modul init
    vector<char> init;
    init.push_back('I');
    init.push_back('N');
    init.push_back('I');
    init.push_back('T');
    init.push_back('M');
    init.push_back(IDMaster);
    sendData(init, slave, spi);
    
    /*for(int i = 6; i < 15; i++)
    {
        StatusRobota r;
        r.id = i;
        r.countNeaktivity = 10;
        r.polohaX = i+5;
        r.polohaY = i+6;
        r.aktivniZadost = false;
        r.countZadostOLokalizaci = 0;
        robotiStatus.push_back(r);
    }*/

    //signalizace, ze je mbed OK a pripraven
    for(int i = 0; i < 3; i++) {
        redLED = 1;
        wait(0.1);
        redLED = 0;
        wait(0.1);
    }

    //spusteni ochranneho timeru
    timerGeneralError.start();

    //hlavni smycka programu
    while(1) {
        //kontrola proti chybe
        if(timerGeneralError.read() > 5) {
            timerGeneralError.reset();
            pc.printf("\r\n\r\nRESET %c\r\n\r\n", '\0');
            najitDostupneRoboty = true;
            najitDostupneRobotyInit = true;
            robotChcePolohy = false;
            oslovitRobota = false;
            polohaPoslana = false;
            cekamNaOdpovedOdRobota = false;
        }

        //kontrola, zda se prenos poloh robotu zdaril
        if(timerOdesilaniPoloh.read_ms() > 500 && robotChcePolohy) {
            pc.printf("\r\nPrenos poloh se nezdaril %c", '\0');
            robotChcePolohy = false;
            polohaPoslana = false;
            timerOdesilaniPoloh.stop();
            timerOdesilaniPoloh.reset();
            cekamNaOdpovedOdRobota = false;
        }

        //vypis informaci
        if(pc.readable()) {
            char serialIn = pc.getc();
            if(serialIn == 'I') {
                pc.printf("\r\n\r\nV pameti %i robotu %c", robotiStatus.size(), '\0');

                //vypis aktivnich a neaktivnich robotu
                vector<char> aktivni;
                vector<char> neaktivni;
                for(int i = 0; i < robotiStatus.size(); i++) {
                    if(robotiStatus.at(i).countNeaktivity < 5) {
                        aktivni.push_back(robotiStatus.at(i).id);
                    } else {
                        neaktivni.push_back(robotiStatus.at(i).id);
                    }
                }

                pc.printf("\r\n\r\nAktivni roboti: ");
                for(int i = 0; i < aktivni.size(); i++) {
                    pc.printf("%c ", aktivni.at(i));
                }
                pc.printf("\r\nNeaktivni roboti: ");
                for(int i = 0; i < neaktivni.size(); i++) {
                    pc.printf("%c ", neaktivni.at(i));
                }

                /*pc.printf("\r\n\r\nPolohy robotu: ");
                for(int i = 0; i < robotiStatus.size(); i++) {
                    pc.printf("\r\n   robot %c: %i, %i", robotiStatus.at(i).id, robotiStatus.at(i).polohaX, robotiStatus.at(i).polohaY);
                }*/
            }
        }

        //procedura pro nalazeni dostupnych robotu
        if(najitDostupneRoboty && !cekamNaOdpovedOdRobota && !robotChcePolohy) {
            if(najitDostupneRobotyInit) {
                najitDostupneRobotyInit = false;
                timerOdesilaniPoloh.reset();  //pretizeni timeru
                timerOdesilaniPoloh.start();
                vector<char> data;
                data.push_back('M');
                data.push_back(IDMaster);
                data.push_back('R');
                data.push_back('X');
                sendData(data, slave, spi);
                pc.printf("\r\nHledam roboty %c",'\0');
            }

            if(timerOdesilaniPoloh.read_ms() > 1000) {
                timerOdesilaniPoloh.stop();
                timerOdesilaniPoloh.reset();
                najitDostupneRoboty = false;
            }
        } else if(robotiStatus.size() > 0) {
            //obsluha robotu
            if(oslovitRobota == true && !polohaPoslana && !robotChcePolohy) {
                oslovitRobota = false;

                aktualniRobot++;        //dalsi robot
                if(aktualniRobot >= robotiStatus.size()) {
                    aktualniRobot = 0;
                }

                //kontrola, zda je robot aktivni
                if(robotiStatus.at(aktualniRobot).countNeaktivity >= 5) {
                    oslovitRobota = true;  //vyzvu dalsiho robota
                } else {
                    //dotaz robota na stav
                    vector<char> data;
                    data.push_back('M');
                    data.push_back(IDMaster);
                    data.push_back('R');
                    data.push_back(robotiStatus.at(aktualniRobot).id);
                    data.push_back('W');

                    sendData(data, slave, spi);     //posilam do IQRF modulu
                    pc.printf("\r\n\r\nDotazuji se robota %c %c", robotiStatus.at(aktualniRobot).id, '\0');
                    cekamNaOdpovedOdRobota = true;
                    //resetuji ochranny timer
                    timerGeneralError.reset();
                }

                //navysim pocitadlo poctu cyklu lokalizace
                if(robotiStatus.at(aktualniRobot).aktivniZadost) {
                    robotiStatus.at(aktualniRobot).countZadostOLokalizaci++;
                }
            } else if(robotChcePolohy && !polohaPoslana && !cekamNaOdpovedOdRobota) {   //posilam polohy vsech robotu

                vector<char> data;
                data.push_back('M');
                data.push_back(IDMaster);
                data.push_back('R');
                data.push_back(IDRobotaCoChcePolohy);
                data.push_back('G');

                if(prenosPolohInit) { //start
                    prenosPolohInit = false;
                    polohPoslano = 0;
                    data.push_back((char)robotiStatus.size());
                    pc.printf("\r\nPosilam robotovi %c, ze mu poslu %i poloh %c", IDRobotaCoChcePolohy, robotiStatus.size(), '\0');
                }
                int deset = polohPoslano + 10;
                for(int i = polohPoslano; i < deset; i++) {
                    if(i < robotiStatus.size()) {
                        //prilozim polohu daneho robota
                        data.push_back(robotiStatus.at(polohPoslano).id);
                        vector<char> x = ushortshortToTwoChars(robotiStatus.at(polohPoslano).polohaX);
                        vector<char> y = ushortshortToTwoChars(robotiStatus.at(polohPoslano).polohaY);
                        for(int j = 0; j < 2; j++) {
                            data.push_back(x.at(j));
                        }
                        for(int j = 0; j < 2; j++) {
                            data.push_back(y.at(j));
                        }

                        //pc.printf("\r\nPosilam polohu robota %c: %i, %i %c", robotiStatus.at(polohPoslano).id, robotiStatus.at(polohPoslano).polohaX, robotiStatus.at(polohPoslano).polohaY, '\0');

                        polohPoslano++;
                    } else {
                        i = deset;
                        data.push_back('E');
                        pc.printf("\r\nPosilam robotovi %c, ze jsem mu prenesl vsechny polohy %c", IDRobotaCoChcePolohy, '\0');
                        robotChcePolohy = false;
                    }
                }
                //posilam do IQRF modulu
                sendData(data, slave, spi);
                timerOdesilaniPoloh.reset();
                polohaPoslana = true;
            }
        }

        //zkontroluji, zda se nejaky robot nesnazi lokalizovat jiz moc dlouho a kdytak odstranim jeho zadost a spocitam pocet aktivnich zadosti
        int pocetZadostiOLokalizaci = 0;
        for(int i = 0; i < robotiStatus.size(); i++) {
            if(robotiStatus.at(i).countZadostOLokalizaci > 30) {
                pc.printf("mazu zadost robota %c, protoze se snazi lokalizovat prilis dlouho %c", robotiStatus.at(i).id, '\0');
                robotiStatus.at(i).aktivniZadost = false;
                robotiStatus.at(i).countZadostOLokalizaci = 0;
            }
            if(robotiStatus.at(i).aktivniZadost) {
                pocetZadostiOLokalizaci++;
            }
        }

        //signalizace aktivnich zadosti o lokalizaci
        if(pocetZadostiOLokalizaci > 0) {
            greenLED = 1;
        } else {
            greenLED = 0;
            blueLED = 0;
        }

        //obsluha slave vezi
        if(pocetZadostiOLokalizaci > 0) {
            if(!pulzVObehu && !vysilamSekvenci) {
                blueLED = 1;
                timerVysilaniSekvence.reset();
                timerVysilaniSekvence.start();
                vysilamSekvenci = true;
            }
        }

        if(timerVysilaniSekvence.read_ms() > dobaLokalizacniSekvence && !pulzVObehu && vysilamSekvenci) {
            vysilamSekvenci = false;
            blueLED = 0;
            timerVysilaniSekvence.stop();
            timerVysilaniSekvence.reset();
            if(pocetZadostiOLokalizaci > 0 && pocetSlaveVezi > 0) {
                pulzVObehu = true;
                next = 1;
                wait_ms(1);
                next = 0;
                timerVysilaniSekvence.reset();
                timerVysilaniSekvence.start();
            }
        }

        if(timerVysilaniSekvence.read_ms() > dobaLokalizacniSekvence*(pocetSlaveVezi+1) && pulzVObehu && !vysilamSekvenci) {
            timerVysilaniSekvence.stop();
            timerVysilaniSekvence.reset();
            pc.printf("\r\n\r\n RING PRERUSEN %c", '\0');
            pulzVObehu = false;
        }

        //pokud jsou pripravena data v IQRF modulu
        if(IQRFData == true) {
            IQRFData = false;

            //dotaz na status IQRF modulu
            char SPI_status = checkSPIStatus(slave, spi);
            SPI_status = checkSPIStatus(slave, spi);

            if(SPI_status >= 64 && SPI_status <= 127) {                     //delka zpravy musi byt v urcitem rozmezi
                vector<char> response = readData(SPI_status, slave, spi);   //prectu data z IQRF modulu
                if(response.size() >= 5 && response.at(0)=='R' && response.at(2)=='M' && response.at(3)== IDMaster) {
                    if(response.at(4) == 'N') {     //robot zpravu neprijal
                        pc.printf("\r\nrobot %c zpravu neprijal: ", response.at(1));
                        for(int i = 0; i < robotiStatus.size(); i++) {  //navysim count neaktivity
                            if(robotiStatus.at(i).id == response.at(1)) {
                                robotiStatus.at(i).countNeaktivity++;
                            }
                        }
                        if(polohaPoslana && IDRobotaCoChcePolohy == response.at(1)) {
                            pc.printf("poloha robota %c", '\0');
                            robotChcePolohy = false;
                            polohaPoslana = false;
                            if(!robotChcePolohy) {
                                timerOdesilaniPoloh.stop();
                            }
                            timerOdesilaniPoloh.reset();
                        } else {
                            pc.printf("dotaz na stav %c", '\0');
                        }
                        cekamNaOdpovedOdRobota = false;
                    } else if(response.at(4) == 'O') { //robot zpravu prijal
                        pc.printf(" ACK %c", '\0');
                        //pc.printf("\r\nrobot %c zpravu prijal: ", response.at(1));
                        for(int i = 0; i < robotiStatus.size(); i++) {  //vynuluji count neaktivity
                            if(robotiStatus.at(i).id == response.at(1)) {
                                robotiStatus.at(i).countNeaktivity = 0;
                            }
                        }
                        if(polohaPoslana && IDRobotaCoChcePolohy == response.at(1)) {
                            //pc.printf(" poloha robota");
                            polohaPoslana = false;

                            if(!robotChcePolohy) {
                                timerOdesilaniPoloh.stop();
                            }
                            timerOdesilaniPoloh.reset();

                        } else {
                            //pc.printf("dotaz, zda neco nechce");
                        }
                    } else                         if(response.at(4) == 'H') {  //robot odpovida
                        timerOdesilaniPoloh.reset();

                        bool alreadyKnown = false;
                        for(int i = 0; i < robotiStatus.size(); i++) {
                            if(robotiStatus.at(i).id == response.at(1)) {//robota jiz znam, vynuluji count neaktivity
                                alreadyKnown = true;
                                robotiStatus.at(i).countNeaktivity = 0;
                                pc.printf("\r\nRobot %c nalezen a jiz znam %c", response.at(1), '\0');
                            }
                        }

                        if(!alreadyKnown) {            //robota jeste neznam, pridam ho do seznamu
                            StatusRobota s;
                            s.id = response.at(1);
                            s.polohaX = 0;
                            s.polohaY = 0;
                            s.countNeaktivity = 0;
                            s.countZadostOLokalizaci = 0;
                            s.aktivniZadost = false;
                            robotiStatus.push_back(s);
                            pc.printf("\r\nRobot %c nalezen a pridan %c", response.at(1), '\0');
                        }
                    } else if(response.at(4) == 'A') {  //robot nema zadnou zadost
                        cekamNaOdpovedOdRobota = false;
                        pc.printf("\r\nrobot %c nema zadnou zadost %c", response.at(1), '\0');
                    } else if(response.at(4) == 'L') {  //robot zada o lokalizaci
                        cekamNaOdpovedOdRobota = false;

                        for(int i = 0; i < robotiStatus.size(); i++) {
                            if(robotiStatus.at(i).id == response.at(1)) {
                                if(robotiStatus.at(i).aktivniZadost) {
                                    pc.printf("\r\nRobot %c jiz ma zadost, lokalizace probiha %c", response.at(1), '\0');
                                }
                                //pokud ne, pridam jeho zadost na seznam
                                else {
                                    pc.printf("\r\nRobot %c zada o lokalizaci %c", response.at(1), '\0');
                                    robotiStatus.at(i).aktivniZadost = true;
                                }
                            }
                        }
                    } else if(response.at(4) == 'P') { //robot posila svou polohu
                        cekamNaOdpovedOdRobota = false;
                        pc.printf("\r\nRobot %c posila svou polohu: ", response.at(1));

                        if(response.size() == 10) {
                            for(int i = 0; i < robotiStatus.size(); i++) {
                                if(robotiStatus.at(i).id == response.at(1)) {

                                    //poloha X
                                    vector<char> x;
                                    for(int j = 6; j <= 7; j++) {
                                        x.push_back(response.at(j));
                                    }
                                    robotiStatus.at(i).polohaX = twoCharsToUShort(x);

                                    //poloha Y
                                    vector<char> y;
                                    for(int j = 8; j <= 9; j++) {
                                        y.push_back(response.at(j));
                                    }
                                    robotiStatus.at(i).polohaY = twoCharsToUShort(y);

                                    //terminal
                                    pc.printf("%i, %i %c", robotiStatus.at(i).polohaX, robotiStatus.at(i).polohaY, '\0');

                                    //odstranim z aktivnich zadosti a vynuluji pocitadlo
                                    robotiStatus.at(i).aktivniZadost = false;
                                    robotiStatus.at(i).countZadostOLokalizaci = 0;
                                    //pc.printf("mazu zadost o lokalizaci robota %c", robotiStatus.at(i));
                                }
                            }
                        } else {    //chyba pri prenosu polohy - spatny format
                            pc.printf("\r\nPrijem polohy robota - spatny format %c", '\0');
                        }
                    } else if(response.at(4) == 'G') {  //robot zada o polohy robotu
                        cekamNaOdpovedOdRobota = false;
                        pc.printf("\r\nrobot %c zada o polohy robotu %c", response.at(1), '\0');
                        prenosPolohInit = true;
                        robotChcePolohy = true;
                        polohaPoslana = false;
                        IDRobotaCoChcePolohy = response.at(1);
                        timerOdesilaniPoloh.reset();
                        timerOdesilaniPoloh.start();
                    } else {          //robot ma neznamy pozadavek
                        cekamNaOdpovedOdRobota = false;
                        pc.printf("\r\nrobot %c ma neznamy pozadavek %c", response.at(1), '\0');
                    }
                }
            }
        }
        if(robotiStatus.size() <= 0 && !najitDostupneRoboty) {
            pc.printf("\r\nzadni roboti %c", '\0');
            wait_ms(500);
            sleep();
        }
    }
}