#include "func.h"

char checkSPIStatus(DigitalOut slave, SPI spi)
{
    slave=0;
    wait_us(10);
    spi.write(0x00);
    wait_us(60);
    char response = spi.write(0x00);
    //pc.printf("SPI check: %02x\r\n",response);
    wait_us(10);
    slave=1;

    return response;
}

char getCRC(vector<char> data)
{
    char crc = data.at(0)^0x5F;
    for(int i = 1; i < data.size(); i++) {
        crc = crc ^ data.at(i);
    }
    //printf("CRC %02x\r\n", crc);
    return crc;
}

vector<char> sendData(vector<char> message, DigitalOut slave, SPI spi)
{
    int sizeOfMessage = message.size();

    //pc.printf("delka dat %i\n\r", sizeOfMessage);
    if(sizeOfMessage > 127) {
        //error
        //error
        wait_ms(50);
        Serial pc(USBTX, USBRX);
        pc.printf("\n\r");
        pc.printf("posilani dat do IQRF modulu: data nemaji spravnou delku");
        
        vector<char> nullData;
        return nullData;
    }


    vector<char> dataToSend;
    dataToSend.push_back(0xF0);                 //SPI_CMD: Data read/write
    dataToSend.push_back(128+sizeOfMessage);    //PTYPE: CTYPE 1 + SPIDLEN
    for(int i = 0; i < sizeOfMessage; i++) {
        dataToSend.push_back(message.at(i));    //vlastni zprava
    }

    char crc = getCRC(dataToSend);
    dataToSend.push_back(crc);                  //CRCM
    dataToSend.push_back(0x00);                 //SPI_CHECK

    vector<char> response;
    slave=0;
    wait_us(10);
    for(int i = 0; i < dataToSend.size(); i++) {
        char tmp = spi.write(dataToSend.at(i));
        response.push_back(tmp);
        wait_us(60);
    }

    wait_us(10);
    slave = 1;


    //CRCS a SPI_STATUS kontrola
    vector<char> dataToCRCS;
    dataToCRCS.push_back(128+sizeOfMessage);
    for(int i = 2; i < response.size()-2; i++) {
        dataToCRCS.push_back(response.at(i));
    }
    char crcs = getCRC(dataToCRCS);


    if(response.at(response.size()-1) != 0x3F) {
        wait_ms(50);
        Serial pc(USBTX, USBRX);
        pc.printf("\n\r");
        pc.printf("posilani dat do IQRF modulu: CRCM ERROR");
        vector<char> nullData;
        return nullData;
    }

    if(response.at(response.size()-2) != crcs) {
        wait_ms(50);
        Serial pc(USBTX, USBRX);
        pc.printf("\n\r");
        pc.printf("posilani dat do IQRF modulu: CRCS ERROR");
        vector<char> nullData;
        return nullData;
    }


    return response;
}

vector<char> readData(char length, DigitalOut slave, SPI spi)
{
    int size;
    if(length == 64) {
        size = 64;
    } else if(length > 64 && length <= 127) {
        size = length - 64;
    } else {
        //error
        wait_ms(50);
        Serial pc(USBTX, USBRX);
        pc.printf("\n\r");
        pc.printf("prijimani dat z IQRF modulu: data nemaji spravnou delku");
        
        vector<char> nullData;
        return nullData;
    }
    vector<char> dataToSend;
    dataToSend.push_back(0xF0);         //SPI_CMD: Data read/write
    dataToSend.push_back(size);         //PTYPE: CTYPE 0 + SPIDLEN
    for(int i = 0; i < size; i++) {
        dataToSend.push_back(0x00);     //dummy (to generate master clock only)
    }
    char crc = getCRC(dataToSend);
    dataToSend.push_back(crc);          //CRCM
    dataToSend.push_back(0x00);         //SPI_CHECK

    vector<char> response;
    slave=0;
    wait_us(10);
    for(int i = 0; i < dataToSend.size(); i++) {
        char tmp = spi.write(dataToSend.at(i));
        response.push_back(tmp);
        wait_us(60);
    }

    wait_us(10);
    slave = 1;

    //CRCS a SPI_STATUS kontrola
    vector<char> dataToCRCS;
    dataToCRCS.push_back(size);
    for(int i = 2; i < response.size()-2; i++) {
        dataToCRCS.push_back(response.at(i));
    }
    char crcs = getCRC(dataToCRCS);


    if(response.at(response.size()-1) != 0x3F) {
        wait_ms(50);
        Serial pc(USBTX, USBRX);
        pc.printf("\n\r");
        pc.printf("prijimani dat z IQRF modulu: CRCM ERROR");
    }

    if(response.at(response.size()-2) != crcs) {
        wait_ms(50);
        Serial pc(USBTX, USBRX);
        pc.printf("\n\r");
        pc.printf("prijimani dat z IQRF modulu: CRCS ERROR");
    }
     
    //odstranim prvni 2 Byty
    response.erase(response.begin(), response.begin() + 2);
    //odstranim posledni 2 Byty
    response.erase(response.end() - 2, response.end());

    return response;
}

void vypis(vector<char> data, int typ)
{
    Serial pc(USBTX, USBRX);
    if(typ == 0) {
        pc.printf("Prijimam: ");
    } else if(typ == 1) {
        pc.printf("\r\nPosilam: ");
    }
    if(data.at(4) == 'P') {
        for(int i = 0; i < 6; i++) {
            pc.printf("%c", data.at(i));
        }
        for(int i = 6; i < data.size(); i++) {
            if(i == 6 | i == 10) {
                pc.printf(" ", data.at(i));
            }
            pc.printf("%x", data.at(i));
        }
    } else {
        for(int i = 0; i < data.size(); i++) {
            pc.printf("%c", data.at(i));
        }
    }
    pc.printf("\r\n");
}

unsigned short twoCharsToUShort(vector<char> input)
{
    unsigned short res = (input[0] << 8) | input[1];

    return res;
}

vector<char> ushortshortToTwoChars(unsigned short input)
{
    vector<char> res;

    res.push_back((input >> 8) & 0xFF);
    res.push_back(input & 0xFF);

    return res;
}

