#include "mbed.h"
#include "func.h"
#include "myClasses.h"
#include <vector>

//---------------- SETUP UZLU--------------------//
char IDRobot = '5';        //ID tohoto robota
//-----------------------------------------------//


//---------------- PROMENNE----------------------//
Timer timerGeneralError;
AnalogIn ain(A4);

//IQRF
SPI spi(D11, D12, D13);             // mosi, miso, sclk
DigitalOut slave(A3);
InterruptIn iqrfInterrupt(D7);

//seriova linka
Serial pc(USBTX, USBRX);
//Serial pc(D1, D0);

//LED
DigitalOut redLED(A2);
DigitalOut blueLED(A1);
DigitalOut greenLED(A0);

//seznam poloh vsech robotu
vector<StatusRobota> polohyRobotu;


bool odpovidatNaHledaniRobotu = true;       //zda se chci prihlasit k masterovi

//zadost o lokalizaci
Ticker tickerLokalizovat;                   //ticker pro lokalizaci
bool chciLokalizovat = false;               //zda chci lokalizovat
bool zadostOLokalizaciPoslana = false;      //zda byla zadost o lokalizaci poslana
bool zadostOLokalizaciPotvrzena = false;    //zda byla zadost o lokalizaci potvrzena
bool poslatSvouPolohu = false;              //zda mam po uspesne lokalizaci poslat svou polohu

//zadost o polohy robotu
Timer timerPolohy;                          //timer pro polohy robotu
Ticker tickerPolohy;                        //ticker pro polohy robotu
bool chciPolohyVsech = false;               //zda chci polohy vse robotu
bool zadostOPolohyPoslana = false;          //zda byla zadost o polohy vsech robotu poslana
bool zadostOPolohyPotvrzena = false;        //zda byla zadost o polohy vsech robotu potvrzena
bool prenosPolohStart = false;              //zda probiha prenos poloh vsech robotu
int pocetPolohKPrijeti = 0;                 //kolik poloh mam prijmout
int pocetPolohPrijato = 0;                  //kolik poloh jsem prijal
//-----------------------------------------------//



//simulace zadosti o lokalizaci
void ChciLokalizovat()
{
    if(chciLokalizovat) {
        chciLokalizovat = false;
        greenLED = 0;
        poslatSvouPolohu = true;
    } else {
        chciLokalizovat = true;
        greenLED = 1;
        zadostOLokalizaciPoslana = false;
        zadostOLokalizaciPotvrzena = false;
        poslatSvouPolohu = false;
    }
}

//simulace zadosti o polohy robotu
void ChciPolohyVsech()
{
    chciPolohyVsech = true;
    blueLED = 1;
    zadostOPolohyPoslana = false;
    zadostOPolohyPotvrzena = false;
}


//IQRF modul ma pripravena data - slouzi k wake up
void iqrfHasData()
{
    //pc.printf("\r\nIQRF data ready");
}

int main()
{
    //seed generatoru pseudonahodnych cisel
    int seed = (int)(ain.read()*1000);
    srand(seed);

    //iniciace tickeru a interrupt pri prijmu dat od IQRF modulu
    tickerLokalizovat.attach(&ChciLokalizovat, 10.0);       //frekvence ze chci lokalizovat
    tickerPolohy.attach(&ChciPolohyVsech, 20.0);            //frekvence ze chci polohy vsech robotu
    iqrfInterrupt.rise(&iqrfHasData);                       //interrupt od IQRF modulu ze ma data

    //SPI setup
    wait(1);
    slave = 1;
    wait(0.1);
    spi.format(8,0);        // Setup:  8bitu, pol = 0, pha = 0 -> rising edge, idle low
    spi.frequency(100000);  //Hz

    //IQRF modul init
    vector<char> init;
    init.push_back('I');
    init.push_back('N');
    init.push_back('I');
    init.push_back('T');
    init.push_back('R');
    init.push_back(IDRobot);
    sendData(init, slave, spi);

    //signalizace, ze je mbed OK a pripraven
    for(int i = 0; i < 3; i++) {
        redLED = 1;
        wait(0.1);
        redLED = 0;
        wait(0.1);
    }

    //spusteni ochranneho timeru
    timerGeneralError.start();

    //hlavni smycka programu
    while(1) {
        //kontrola, zda se prenos poloh robotu zdaril
        if(timerPolohy.read_ms() > 1000) {
            timerPolohy.stop();
            timerPolohy.reset();
            chciPolohyVsech = true;
            blueLED = 0;
            if(chciPolohyVsech) {
                zadostOPolohyPoslana = false;
                zadostOPolohyPotvrzena = false;
                prenosPolohStart = false;
            }
            pocetPolohPrijato = 0;
            pc.printf("\r\nPrenos poloh se nezdaril %c", '\0');
        }

        //dotaz na status IQRF modulu
        char SPI_status = checkSPIStatus(slave, spi);
        SPI_status = checkSPIStatus(slave, spi);

        //IQRF modul ma pripravena data
        if(SPI_status >= 64 && SPI_status <= 127) {
            //prectu data z IQRF modulu
            vector<char> response = readData(SPI_status, slave, spi);
            if(response.size() >= 5) {
                if(response.at(0)=='M' && response.at(2)=='R' && response.at(3)== IDRobot) {
                    timerGeneralError.reset();
                    if(response.at(4) == 'N') {     //master zpravu neprijal
                        pc.printf("\r\nmaster %c zpravu neprijal: ", response.at(1));
                        if(chciLokalizovat && zadostOLokalizaciPoslana && !zadostOLokalizaciPotvrzena) {
                            zadostOLokalizaciPoslana = false;
                            zadostOLokalizaciPotvrzena = false;
                            pc.printf("zadost o lokalizaci %c", '\0');
                        } else if(chciPolohyVsech && zadostOPolohyPoslana && !zadostOPolohyPotvrzena) {
                            zadostOPolohyPoslana = false;
                            zadostOPolohyPotvrzena = false;
                            pc.printf("zadost o polohy robotu %c", '\0');
                        } else if(poslatSvouPolohu) {
                            pc.printf("moje poloha %c", '\0');
                        } else {
                            pc.printf("zadna zadost %c", '\0');
                        }
                    } else if(response.at(4) == 'O') { //master zpravu prijal
                        pc.printf(" ACK %c", '\0');
                        //pc.printf("\r\nmaster %c zpravu prijal: ", response.at(1));
                        if(chciLokalizovat && zadostOLokalizaciPoslana && !zadostOLokalizaciPotvrzena) {
                            zadostOLokalizaciPoslana = false;
                            zadostOLokalizaciPotvrzena = true;
                            //pc.printf("zadost o lokalizaci");
                        } else if(chciPolohyVsech && zadostOPolohyPoslana && !zadostOPolohyPotvrzena) {
                            zadostOPolohyPoslana = false;
                            zadostOPolohyPotvrzena = true;
                            //pc.printf("zadost o polohy vsech robotu");
                        } else if(poslatSvouPolohu) {
                            poslatSvouPolohu = false;
                            //pc.printf("moje poloha");
                        } else {
                            //pc.printf("nic nechci");
                        }
                    } else {                        //master posila zpravu
                        vector<char> data;
                        data.push_back('R');
                        data.push_back(IDRobot);
                        data.push_back('M');
                        data.push_back(response.at(1));

                        if(response.at(4) == 'W') { //master se pta, zda neco nechci
                            pc.printf("\r\n\r\nMaster %c se dotazuje na stav %c", response.at(1), '\0');


                            if(chciLokalizovat && !zadostOLokalizaciPotvrzena) {    //zadost o lokalizaci
                                zadostOLokalizaciPoslana = true;
                                zadostOLokalizaciPotvrzena = false;
                                data.push_back('L');
                                pc.printf("\r\nodpovidam M%c: zadost o lokalizaci %c", data.at(3), '\0');
                            } else if(chciPolohyVsech && !zadostOPolohyPotvrzena) { //zadost o polohy robotu
                                zadostOPolohyPoslana = true;
                                zadostOPolohyPotvrzena = false;
                                timerPolohy.start();
                                data.push_back('G');
                                pc.printf("\r\nodpovidam M%c: zadost o polohy robotu %c", data.at(3), '\0');
                            } else if(poslatSvouPolohu) {   //uspesna lokalizace, posilam svou polohu
                                data.push_back('P');

                                //ID robota
                                data.push_back(IDRobot);

                                //poloha
                                int x = (((int)IDRobot+5))*11;
                                vector<char> vecX = ushortshortToTwoChars(x);
                                for(int i = 0; i < 2; i++) {
                                    data.push_back(vecX.at(i));
                                }

                                int y = (((int)IDRobot+2))*7;
                                vector<char> vecY = ushortshortToTwoChars(y);
                                for(int i = 0; i < 2; i++) {
                                    data.push_back(vecY.at(i));
                                }

                                pc.printf("\r\nodpovidam M%c: uspesna lokalizace, ma poloha %i, %i %c", data.at(3), x, y, '\0');
                            } else {    //zadna zadost
                                data.push_back('A');
                                pc.printf("\r\nodpovidam M%c: zadna zadost %c", data.at(3), '\0');
                            }
                            sendData(data, slave, spi); //poslu do IQRF modulu

                        } else if( response.at(4) == 'G') { //master posila polohy robota
                            int responseSize = response.size();
                            if(responseSize >= 6) {
                                int pocatekPoloh;
                                if(!prenosPolohStart) { //zacatek prenosu
                                    prenosPolohStart = true;
                                    timerPolohy.reset();
                                    pocetPolohKPrijeti = (int)response.at(5);
                                    pc.printf("\r\nMaster %c posila %i poloh %c", response.at(1), pocetPolohKPrijeti, '\0');
                                    pocatekPoloh = 6;
                                }
                                else {
                                    pocatekPoloh = 5;
                                    }
                                for(int i = pocatekPoloh; i < responseSize; i += 5) {
                                    if(responseSize >= i+5) {
                                        pocetPolohPrijato++;
                                        timerPolohy.reset();

                                        //vymazu ze seznamu poloh stary zaznam
                                        for(int j = 0; j < polohyRobotu.size(); j++) {
                                            if(polohyRobotu.at(j).id == response.at(i));
                                            {
                                                polohyRobotu.erase(polohyRobotu.begin() + j);
                                            }
                                        }

                                        //ulozim do seznamu poloh
                                        StatusRobota poloha;
                                        poloha.id = response.at(i);

                                        //ukladam polohu robota
                                        vector<char> x;
                                        x.push_back(response.at(i+1));
                                        x.push_back(response.at(i+2));
                                        poloha.polohaX = twoCharsToUShort(x);

                                        vector<char> y;
                                        y.push_back(response.at(i+3));
                                        y.push_back(response.at(i+4));
                                        poloha.polohaY = twoCharsToUShort(y);

                                        polohyRobotu.push_back(poloha);
                                        pc.printf("\r\nPrijimam polohu robota %c %i, %i", poloha.id, poloha.polohaX, poloha.polohaY);

                                    } else if(response.at(i) == 'E') {
                                        prenosPolohStart = false;
                                        timerPolohy.stop();
                                        timerPolohy.reset();
                                        pc.printf("\r\nMaster %c poslal %i poloh %c", response.at(1), pocetPolohPrijato, '\0');
                                        if(pocetPolohKPrijeti != pocetPolohPrijato) {
                                            pc.printf("\r\nMel jsem prijmout %i poloh, ale prijal jsem jich %i %c", pocetPolohKPrijeti, pocetPolohPrijato, '\0');
                                            chciPolohyVsech = true;
                                            blueLED = 1;
                                            zadostOPolohyPoslana = false;
                                            zadostOPolohyPotvrzena = false;
                                        } else {
                                            chciPolohyVsech = false;
                                            blueLED = 0;
                                        }
                                        pocetPolohKPrijeti = 0;
                                        pocetPolohPrijato = 0;
                                    } else {
                                        pc.printf("\r\nPrenos poloh - spatny format %c", '\0');
                                    }
                                }
                            } else {
                                pc.printf("\r\nPrenos poloh - spatny format %c", '\0');
                            }
                        }
                    }
                }
            } else if(response.size() == 4 && response.at(0)=='M' && response.at(2)=='R' && response.at(3)== 'X' && odpovidatNaHledaniRobotu) { //master hleda roboty

                vector<char> data;
                data.push_back('R');
                data.push_back(IDRobot);
                data.push_back('M');
                data.push_back(response.at(1));
                data.push_back('H');

                wait_ms(rand()%500);    //cekam nahodnou dobu 0-500ms
                sendData(data, slave, spi);

                pc.printf("\r\nMaster %c hleda roboty, odpovidam %c", response.at(1), '\0');
                odpovidatNaHledaniRobotu = false;
            }
        }
        //kontrola aktivity a proti necekane chybe
        if(timerGeneralError.read() > 5) {
            timerGeneralError.reset();
            pc.printf("\r\n\r\nRESET %c\r\n\r\n", '\0');
            zadostOPolohyPoslana = false;
            zadostOPolohyPotvrzena = false;
            prenosPolohStart = false;
            zadostOLokalizaciPoslana = false;
            zadostOLokalizaciPotvrzena = false;
            odpovidatNaHledaniRobotu = true;
        }

        if(!prenosPolohStart) {  //sleep
            sleep();
        }
    }
}
