package com.example.mrtea.hc06_blink;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.opengl.Visibility;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.widget.TextView;
import android.widget.EditText;
import android.widget.Button;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Set;
import java.util.UUID;

public class MainActivity extends Activity
{
    TextView terminal;
    Button myBtn;
    Button btnRefresh;
    Button btnInfo;
    boolean run = false;
    boolean connected = false;
    BluetoothAdapter mBluetoothAdapter;
    BluetoothSocket mmSocket;
    BluetoothDevice mmDevice;
    OutputStream mmOutputStream;
    InputStream mmInputStream;
    Thread workerThread;
    byte[] readBuffer;
    int readBufferPosition;
    volatile boolean stopWorker;

    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        terminal = (TextView)findViewById(R.id.terminal);
        myBtn = (Button)findViewById(R.id.btnRunPause);
        btnRefresh= (Button)findViewById(R.id.btnRefresh);
        btnInfo = (Button)findViewById(R.id.btnInfo);

        terminal.setText("Openning...");
        myBtn.setText("Run");
        myBtn.setOnClickListener(
                new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        if(connected) {
                            run = !run;
                            if (run) {
                                myBtn.setText("Pause");
                                run = true;
                                btnInfo.setVisibility(View.VISIBLE);
                                beginListenForData();

                            } else {
                                myBtn.setText("Run");
                                btnInfo.setVisibility(View.INVISIBLE);
                            }
                        }
                    }
                }
        );

        btnRefresh.setOnClickListener(
                new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        btnRefresh.setVisibility(View.INVISIBLE);
                        try {
                            terminal.setText("Opening..." + "\n" + terminal.getText());
                            findBT();
                            new LongOperation().execute("");
                        }
                        catch (Exception e)
                        {
                            Log.d("Error", e.toString());
                        }
                    }
                }
        );

        btnInfo.setOnClickListener(
                new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        try {
                            mmOutputStream.write('I');
                        }
                        catch (Exception e)
                        {
                            Log.d("Error", e.toString());
                        }
                    }
                }
        );

        try {
            findBT();
            new LongOperation().execute("");
        }
        catch (Exception e)
        {
            Log.d("Error", e.toString());
        }
    }

    void findBT()
    {
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

        if(mBluetoothAdapter == null)
        {
            terminal.setText("No bluetooth adapter available"+ "\n" + terminal.getText());
        }

        if(!mBluetoothAdapter.isEnabled())
        {
            Intent enableBluetooth = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableBluetooth, 0);
        }

        Set<BluetoothDevice> pairedDevices = mBluetoothAdapter.getBondedDevices();
        if(pairedDevices.size() > 0)
        {
            for(BluetoothDevice device : pairedDevices)
            {
                if(device.getName().equals("HC-06"))
                {
                    mmDevice = device;
                    break;
                }
            }
        }
    }

    private class LongOperation extends AsyncTask<String, Void, String> {
        @Override
        protected String doInBackground(String... params) {
            openBT();
            return null;
        }

        @Override
        protected void onPostExecute(String result)
        {
            if(connected) {
                terminal.setText("Bluetooth connected"+ "\n" + terminal.getText());
                btnRefresh.setVisibility(View.INVISIBLE);
                myBtn.setVisibility(View.VISIBLE);
                myBtn.setText("RUN");
                btnInfo.setVisibility(View.INVISIBLE);
                run = false;
                beginListenForData();
            }
            else {
                terminal.setText("Bluetooth not connected"+ "\n" + terminal.getText());
                btnRefresh.setVisibility(View.VISIBLE);
                myBtn.setVisibility(View.INVISIBLE);
                btnInfo.setVisibility(View.INVISIBLE);
            }
        }
    }
    void openBT()
    {
        try {
            UUID uuid = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB"); //Standard SerialPortService ID
            mmSocket = mmDevice.createRfcommSocketToServiceRecord(uuid);
            mmSocket.connect();
            mmOutputStream = mmSocket.getOutputStream();
            mmInputStream = mmSocket.getInputStream();

            connected = true;
        }
        catch (Exception e)
        {
            Log.d("Error", e.toString());
            connected = false;
        }
    }

    void beginListenForData()
    {
        final Handler handler = new Handler();
        final byte delimiter = 10; //This is the ASCII code for a newline character

        stopWorker = false;
        readBufferPosition = 0;
        readBuffer = new byte[1024];

        IntentFilter filter = new IntentFilter(BluetoothDevice.ACTION_ACL_DISCONNECTED);
        BroadcastReceiver mReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                String action = intent.getAction();

                if (BluetoothDevice.ACTION_ACL_DISCONNECTED.equals(action)) {
                    terminal.setText("Not Connected" + "\n" + terminal.getText());
                    run = false;
                    btnRefresh.setVisibility(View.VISIBLE);
                    myBtn.setVisibility(View.INVISIBLE);
                    btnInfo.setVisibility(View.INVISIBLE);
                }
            }
        };
        this.registerReceiver(mReceiver, filter);

        workerThread = new Thread(new Runnable()
        {
            public void run()
            {
                while(!Thread.currentThread().isInterrupted() && run)
                {
                    try
                    {
                        if(mmInputStream != null && connected) {
                            int bytesAvailable = mmInputStream.available();
                            if (bytesAvailable > 0) {
                                byte[] packetBytes = new byte[bytesAvailable];
                                mmInputStream.read(packetBytes);
                                for (int i = 0; i < bytesAvailable; i++) {
                                    byte b = packetBytes[i];
                                    if (b == delimiter) {
                                        byte[] encodedBytes = new byte[readBufferPosition];
                                        System.arraycopy(readBuffer, 0, encodedBytes, 0, encodedBytes.length);
                                        final String data = new String(encodedBytes, "US-ASCII");
                                        readBufferPosition = 0;

                                        handler.post(new Runnable() {
                                            public void run() {
                                                terminal.setText(data + "\n" + terminal.getText());
                                            }
                                        });
                                    } else {
                                        readBuffer[readBufferPosition++] = b;
                                    }
                                }
                            }
                        }
                        else
                        {
                            btnRefresh.setVisibility(View.VISIBLE);
                            myBtn.setVisibility(View.INVISIBLE);
                            btnInfo.setVisibility(View.INVISIBLE);
                            connected = false;
                            terminal.setText("Not Connected" + "\n" + terminal.getText());
                            run = false;
                        }
                    }
                    catch (IOException ex)
                    {
                        terminal.setText("Not Connected" + "\n" + terminal.getText());
                        run = false;
                        btnRefresh.setVisibility(View.VISIBLE);
                        myBtn.setVisibility(View.INVISIBLE);
                        btnInfo.setVisibility(View.INVISIBLE);
                    }
                }
            }
        });

        workerThread.start();
    }
}