﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.IO;
using System.IO.Ports;
using System.Windows.Forms;

namespace SerialPortLogger
{
    public class COMPort
    {
        private string port_name;

        private SerialPort port;
        private string path;
        private bool portOpen;

        /*------------------------------------------------------
        Constructors can be added as needed to take
        additional inputs for more flexibility.
        ------------------------------------------------------*/
        public COMPort(string name)
        {
            port = new SerialPort();
            port_name = name;
            port.PortName = name;
            port.BaudRate = 9600;
            port.Parity = Parity.None;
            port.DataBits = 8;
            port.StopBits = StopBits.One;
            port.Handshake = Handshake.None;
            port.Encoding = Encoding.UTF8;
            portOpen = false;

        }

        public void getInfo()
        {
            this.port.Write("I");
        }

        public string getName()
        {
            return this.port_name;
        }

        /*------------------------------------------------------
        Close the serial port and file.
        ------------------------------------------------------*/
        public bool close()
        {
            if (portOpen)
            {
                try
                {
                    port.DataReceived -= port_DataReceived;
                    port.Close();
                    portOpen = false;
                    return true;
                }
                catch
                {
                    return false;
                }
            }
            return true;
        }

        /*------------------------------------------------------
        Add the serial data received handler, open the file to
        log to, and open the serial port.
        ------------------------------------------------------*/
        public bool open(string path, string name)
        {
            if (!portOpen)
            {
                try {
                    this.path = path;
                    port.Open();
                    portOpen = true;
                    port.DataReceived += port_DataReceived;
                    return true;
                }
                catch
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }

        /*------------------------------------------------------
        Handler for the serial data events.
        ------------------------------------------------------*/
        void port_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                string data = port.ReadExisting();
                if (!string.IsNullOrEmpty(data))
                {
                    //if(data.Contains(Environment.NewLine))
                    {
                        DateTime today = DateTime.Now;
                        string cas = "  cas: " + today.ToString("dd.MM.yyyy HH:mm:ss:fff");
                        //data = data.Insert(data.IndexOf(Environment.NewLine), cas);
                        data = data.Replace("\0", cas + ";");
                    }

                    using (StreamWriter w = File.AppendText(this.path + "log" + this.port_name + ".csv"))
                    {
                        w.Write(data);
                    }
                }
            }
            catch
            {
                
            }
        }
    }
}
