﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using System.IO.Ports;

namespace SerialPortLogger
{
    public partial class Form1 : Form
    {
        private List<COMPort> COMPorts;
        private string location;

        public Form1()
        {
            InitializeComponent();
            this.COMPorts = new List<COMPort>();
            RefreshListAvailable();
            RefreshListOpened();
            this.location = System.Reflection.Assembly.GetEntryAssembly().Location;
            this.location = this.location.Remove(this.location.LastIndexOf("\\") + 1, this.location.Length - this.location.LastIndexOf("\\") - 1);
            setLocStr();
        }

        public void setLocStr()
        {
            string loc = this.location;
            if (loc.Length > 50)
            {
                loc = loc.Substring(0, 50) + "...";
            }
            lblUmisteni.Text = loc;
        }

        public void RefreshListAvailable()
        {
            checkedListBox1.Items.Clear();
            var availablePorts = SerialPort.GetPortNames().ToList();
            foreach (var port in availablePorts)
            {
                checkedListBox1.Items.Add(port);
            }
        }

        public void RefreshListOpened()
        {
            listBox1.Items.Clear();
            foreach (var port in COMPorts)
            {
                listBox1.Items.Add(port.getName());
            }
        }

        private void btnOpen_Click(object sender, EventArgs e)
        {
            lblErrorOpen.Visible = false;
            lblErrorOpen.Text = "";
            var portsToOpen = checkedListBox1.CheckedItems;
            lblZpracovavam.Visible = true;
            this.Refresh();
            foreach (var portToOpen in portsToOpen)
            {
                bool opened = false;
                foreach (var portOpened in COMPorts)
                {
                    if (portToOpen.ToString() == portOpened.getName())
                    {
                        opened = true;
                    }
                }
                if (!opened)
                {
                    var COMPort = new COMPort(portToOpen.ToString());
                    bool complete = COMPort.open(this.location, portToOpen.ToString());
                    
                    if (complete)
                    {
                        COMPorts.Add(COMPort);
                    }
                    else
                    {
                        lblErrorOpen.Text += "Port " + portToOpen.ToString() + " se nepodařilo otevřít." + Environment.NewLine;
                        lblErrorOpen.Visible = true;
                    }
                }
            }
            lblZpracovavam.Visible = false;
            RefreshListOpened();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            lblErrorOpen.Visible = false;
            lblErrorOpen.Text = "";
            var portsToClose = checkedListBox1.CheckedItems;
            lblZpracovavam.Visible = true;
            this.Refresh();
            foreach (var portToClose in portsToClose)
            {
                COMPort port = null;
                bool close = false;

                foreach (var portOpened in COMPorts)
                {
                    if (portToClose.ToString() == portOpened.getName())
                    {
                        close = true;
                        port = portOpened;
                    }
                }
                if (close)
                {
                    bool complete = port.close();
                    if (complete)
                    {
                        COMPorts.Remove(port);
                    }
                    else
                    {
                        lblErrorOpen.Text += "Port " + port.ToString() + " se nepodařilo zavřít." + Environment.NewLine;
                        lblErrorOpen.Visible = true;
                    }
                }


            }
            lblZpracovavam.Visible = false;
            RefreshListOpened();
        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            lblErrorOpen.Visible = false;
            RefreshListAvailable();
        }

        private void btnUmisteni_Click(object sender, EventArgs e)
        {
            using (var fbd = new FolderBrowserDialog())
            {
                fbd.RootFolder = Environment.SpecialFolder.MyComputer;
                fbd.SelectedPath = this.location.Remove(this.location.LastIndexOf("\\") + 1, this.location.Length - this.location.LastIndexOf("\\") - 1);
                DialogResult result = fbd.ShowDialog();

                if (result == DialogResult.OK && !string.IsNullOrWhiteSpace(fbd.SelectedPath))
                {
                    this.location = fbd.SelectedPath + "\\";
                    setLocStr();
                }
            }
        }

        private void btnVypis_Click(object sender, EventArgs e)
        {
            RefreshListOpened();
            foreach (var p in listBox1.Items)
            {
                foreach (var portOpened in COMPorts)
                {
                    if (p.ToString() == portOpened.getName())
                    {
                        portOpened.getInfo();
                    }
                }
            }
        }
    }
}
